@@ -7,7 +7,7 @@
 #'   scoped variant of \code{rec()}, where recoding will be applied only
 #'   to those variables that match the logical condition of \code{predicate}.
 #'
-#' @seealso \code{\link{set_na}} for setting \code{NA} values, \code{\link{replace_na}}
+#' @seealso \code{\link[sjlabelled]{set_na}} for setting \code{NA} values, \code{\link{replace_na}}
 #'            to replace \code{NA}'s with specific value, \code{\link{recode_to}}
 #'            for re-shifting value ranges and \code{\link{ref_lvl}} to change the
 #'            reference level of (numeric) factors.
@@ -62,7 +62,7 @@
 #'     \item{recode pairs}{each recode pair has to be separated by a \code{;}, e.g. \code{rec = "1=1; 2=4; 3=2; 4=3"}}
 #'     \item{multiple values}{multiple old values that should be recoded into a new single value may be separated with comma, e.g. \code{"1,2=1; 3,4=2"}}
 #'     \item{value range}{a value range is indicated by a colon, e.g. \code{"1:4=1; 5:8=2"} (recodes all values from 1 to 4 into 1, and from 5 to 8 into 2)}
-#'     \item{value range for doubles}{for double vectors (with floating points), all values within the specified range are recoded; e.g. \code{1:2.5=1;2.6:3=2} recodes 1 to 2.5 into 1 and 2.6 to 3 into 2, but 2.55 would not be recoded (since it's not included in any of the specified ranges)}
+#'     \item{value range for doubles}{for double vectors (with fractional part), all values within the specified range are recoded; e.g. \code{1:2.5=1;2.6:3=2} recodes 1 to 2.5 into 1 and 2.6 to 3 into 2, but 2.55 would not be recoded (since it's not included in any of the specified ranges)}
 #'     \item{\code{"min"} and \code{"max"}}{minimum and maximum values are indicates by \emph{min} (or \emph{lo}) and \emph{max} (or \emph{hi}), e.g. \code{"min:4=1; 5:max=2"} (recodes all values from minimum values of \code{x} to 4 into 1, and from 5 to maximum values of \code{x} into 2)}
 #'     \item{\code{"else"}}{all other values, which have not been specified yet, are indicated by \emph{else}, e.g. \code{"3=1; 1=2; else=3"} (recodes 3 into 1, 1 into 2 and all other values into 3)}
 #'     \item{\code{"copy"}}{the \code{"else"}-token can be combined with \emph{copy}, indicating that all remaining, not yet recoded values should stay the same (are copied from the original value), e.g. \code{"3=1; 1=2; else=copy"} (recodes 3 into 1, 1 into 2 and all other values like 2, 4 or 5 etc. will not be recoded, but copied, see 'Examples')}
@@ -104,7 +104,7 @@
 #'   mutate(dependency_rev = rec(e42dep, rec = "rev")) %>%
 #'   head()
 #'
-#' # recode 1 to 3 into 4 into 2
+#' # recode 1 to 3 into 1 and 4 into 2
 #' table(rec(efc$e42dep, rec = "min:3=1; 4=2"), useNA = "always")
 #'
 #' # recode 2 to 1 and all others into 2
@@ -177,8 +177,7 @@
 #' # recode only variables that have a value range from 1-4
 #' p <- function(x) min(x, na.rm = TRUE) > 0 && max(x, na.rm = TRUE) < 5
 #' rec_if(efc, predicate = p, rec = "1:3=1;4=2;else=copy")
-#'
-#'
+#' @importFrom insight print_color
 #' @export
 rec <- function(x, ..., rec, as.num = TRUE, var.label = NULL, val.labels = NULL, append = TRUE, suffix = "_r") {
   UseMethod("rec")
@@ -323,29 +322,32 @@ rec_helper <- function(x, recodes, as.num, var.label, val.labels) {
   # remember if NA's have been recoded...
   na_recoded <- FALSE
 
+  # save some sates...
+  reversed_value_labels <- NULL
+  recodes_reversed <- recodes == "rev"
+  n_values <- n_unique(x)
+
   # drop labels when reversing
-  if (recodes == "rev") x <- sjlabelled::drop_labels(x, drop.na = TRUE)
+  if (recodes_reversed) {
+    reversed_value_labels <- sjlabelled::get_labels(
+      x,
+      attr.only = TRUE,
+      values = "n",
+      non.labelled = FALSE,
+      drop.na = TRUE
+    )
+
+    # reverse value labels, but not values
+    r_values <- names(reversed_value_labels)
+    reversed_value_labels <- rev(reversed_value_labels)
+    names(reversed_value_labels) <- r_values
+  }
 
   # get current NA values
   current.na <- sjlabelled::get_na(x)
 
   # do we have a factor with "x"?
   if (is.factor(x)) {
-    # save variable labels before in case we just want
-    # to reverse the order
-    if (is.null(val_lab) && recodes == "rev") {
-      val_lab <-
-        rev(
-          sjlabelled::get_labels(
-            x,
-            attr.only = TRUE,
-            values = NULL,
-            non.labelled = TRUE,
-            drop.na = TRUE
-          )
-        )
-    }
-
     if (is_num_fac(x)) {
       # numeric factors coerced to numeric
       x <- as.numeric(as.character(x))
@@ -367,30 +369,18 @@ rec_helper <- function(x, recodes, as.num, var.label, val.labels) {
   max_val <- max(x, na.rm = T)
 
   # do we have special recode-token?
-  if (recodes == "rev") {
-
+  if (recodes_reversed) {
     # retrieve unique valus, sorted
-    ov <- sort(unique(stats::na.omit(as.vector(x))))
+    ov <- if (!is.null(reversed_value_labels) && n_values <= length(reversed_value_labels))
+      as.numeric(as.vector(names(reversed_value_labels)))
+    else
+      sort(unique(stats::na.omit(as.vector(x))))
 
     # new values should be reversed order
     nv <- rev(ov)
 
     # create recodes-string
     recodes <- paste(sprintf("%i=%i", ov, nv), collapse = ";")
-
-    # when we simply reverse values, we can keep value labels
-    if (is.null(val_lab)) {
-      val_lab <-
-        rev(
-          sjlabelled::get_labels(
-            x,
-            attr.only = TRUE,
-            values = NULL,
-            non.labelled = TRUE,
-            drop.na = TRUE
-          )
-        )
-    }
   }
 
   # we allow direct labelling, so extract possible direct labels here
@@ -458,6 +448,14 @@ rec_helper <- function(x, recodes, as.num, var.label, val.labels) {
     stop(sprintf("?Syntax error in argument \"%s\"", paste(correct_syntax, collapse = "=")), call. = F)
   }
 
+
+  # check for duplicated inputs
+  if (anyDuplicated(sapply(rec_pairs, function(i) i[2])) > 0) {
+    insight::print_color("One or more of the old values are recoded into identical new values.\nPlease check if you correctly specified the recode-pattern,\nelse separate multiple values with comma, e.g.", "red")
+    insight::print_color(" rec=\"a,b,c=1; d,e,f=2\"", "green")
+    insight::print_color(".\n", "red")
+  }
+
   # the new, recoded variable
   new_var <- rep(-Inf, length(x))
 
@@ -571,7 +569,7 @@ rec_helper <- function(x, recodes, as.num, var.label, val.labels) {
 
       } else {
         # else we have numeric values, which should be replaced
-        new_var[which(x == old_val[k])] <- new_val
+        new_var[which(gsub(" ", "", x) == old_val[k])] <- new_val
       }
     }
   }
@@ -579,6 +577,11 @@ rec_helper <- function(x, recodes, as.num, var.label, val.labels) {
   # replace remaining -Inf with NA
   if (any(is.infinite(new_var))) new_var[which(new_var == -Inf)] <- NA
 
+  # if we have no value labels, but we have reversed labels, add them back now
+  if (is.null(val_lab) && !is.null(reversed_value_labels) && n_values <= length(reversed_value_labels)) {
+    val_lab <- reversed_value_labels
+  }
+
   # add back NA labels
   if (!is.null(current.na) && length(current.na) > 0) {
     # add named missings
